//
//  MFAuthService.h
//  MFAuthLib
//
//  Created by MicroFocus on 1/26/17.
//  Copyright © 2017 Microfocus. All rights reserved.
//

#import <Foundation/Foundation.h>
@import SafariServices;

#import "MFServiceConfiguration.h"
#import "MFUtil.h"

//#import <AppAuthLib/AppAuth.h>

@protocol OIDAuthorizationFlowSession;

@class OIDAuthState;

/*! @brief Represents the type of block used as a callback for creating a service configuration from
 a remote OpenID Connect Discovery document.
 @param configuration The service configuration, if available.
 @param error The error if an error occurred.
 */
typedef void (^MFDiscoveryCallback)(MFServiceConfiguration *_Nullable configuration, NSError *_Nullable error);

typedef void (^MFAuthorizationFlowCallback)(id<OIDAuthorizationFlowSession> _Nullable currentAuthFlowSession,
NSError *_Nullable error);

/*! @brief Represents the type of block used as a callback after completion of SignIn.
 @param success The service configuration, if available.
 @param error The error if an error occurred.
 */
typedef void (^MFSignInCompletionCallback)(BOOL success, NSError *_Nullable error);

/*! @brief Represents the type of block used as a callback for network calls in various methods in MFAuthService.
 @param data The response data configuration, if available.
 @param response The response of the network call.
 @param error The error if an error occurred.
 */
typedef void (^MFResponseCallback)(NSData *_Nullable data, NSURLResponse *_Nullable response, NSError *_Nullable error);

/*! @brief Represents the type of block used as a callback with call to UserInfo endpoint.
 @param dictUserData User data disctionary as fetched from the userinfo endpoint.
 @param error The error if an error occurred.
 */
typedef void (^MFUserInfoCallback)(NSDictionary *_Nullable dictUserData, NSError *_Nullable error);

/*! @brief Represents a block used to call an action with a fresh access token.
 @param accessToken A valid access token if available.
 @param idToken A valid ID token if available.
 @param error The error if an error occurred.
 */
typedef void (^MFAuthStateAction)(NSString *_Nullable accessToken, NSString *_Nullable idToken, NSError *_Nullable error);



/*! @brief Performs various OAuth and OpenID Connect related calls via the user agent or \NSURLSession.
 */
@interface MFAuthService : NSObject <SFSafariViewControllerDelegate>

@property(nonatomic, readonly, nullable) OIDAuthState *authState;

@property(nonatomic, readonly, nullable) OIDServiceConfiguration *OIDConfig;

/*! @brief The service's configuration.
 @remarks Each authorization service is initialized with a configuration. This configuration
 specifies how to connect to a particular OAuth provider. Clients should use separate
 authorization service instances for each provider they wish to integrate with.
 Configurations may be created manually, or via an OpenID Connect Discovery Document.
 */
@property(nonatomic, readonly, nullable) MFServiceConfiguration *config;


/*! @brief Convenience method for creating an authorization service configuration from an OpenID
 Connect compliant issuer URL.
 @param discoveryUrl The service provider's OpenID Connect issuer.
 @param completionCallback A block which will be invoked when the authorization service configuration has
 been created, or when an error has occurred.
 @see https://openid.net/specs/openid-connect-discovery-1_0.html
 */
+ (void) discoverServiceConfiguration:(nonnull NSURL *) discoveryUrl  completion:(nonnull MFDiscoveryCallback) completionCallback;

/*! @brief Remove an existing Keychain entry with tokens.
 This can be used for e.g. when the orphaned tokens needs to be explicitly removed at the first run of the app.
 */
+ (void) removeKeychainEntry;
    

- (nullable instancetype) init NS_UNAVAILABLE;


/*! @brief Initialize service with service's configuration.
 This is the preferred way to initialize.
 @param serviceConfig Service Configuration to initialize with.
 */
- (nullable instancetype) initWithServiceConfig:(nonnull MFServiceConfiguration *) serviceConfig
    NS_DESIGNATED_INITIALIZER;

/*! @brief Initialize auth service with endpoints.
 @param authorizationUrl OAuth authorization endpoint
 @param tokenUrl Oauth token endpoint
 @param revocationUrl Token revocation endpoint
 @param userinfoUrl UserInfo endpoint.
 */
- (nullable instancetype) initWithAuthzEndpoint:(nonnull NSURL *) authorizationUrl
                                  tokenEndpoint:(nonnull NSURL *) tokenUrl
                             revocationEndpoint:(nullable NSURL *) revocationUrl
                               userinfoEndpoint:(nullable NSURL *) userinfoUrl;

/*! @brief Another Initializer with service endpoints.
 @param authorizationUrl OAuth authorization endpoint
 @param tokenUrl Oauth token endpoint
 @param revocationUrl Token revocation endpoint
 @param userinfoUrl UserInfo endpoint.
 @param browserLogoutUrl Url to logout user session from the device system browser or @C SFSafariViewController
 */
- (nullable instancetype) initWithAuthzEndpoint:(nonnull NSURL *) authorizationUrl
                                  tokenEndpoint:(nonnull NSURL *) tokenUrl
                             revocationEndpoint:(nullable NSURL *) revocationUrl
                               userinfoEndpoint:(nullable NSURL *) userinfoUrl
                          browserLogoutEndpoint:(nullable NSURL *) browserLogoutUrl;

/*! @brief Initialize auth service with service's base Url.
 This initializer assumes NAM authorization Identity Provider and builds appropriate endpoints.
 @param providerBaseUrl NAM IDP base Url e.g. https://www.idp.com
 */
- (nullable instancetype) initWithProvider:(nonnull NSURL *) providerBaseUrl;

/*! @brief Initialize auth service with service's issuer Url.
 This initializer assumes NAM authorization Identity Provider and builds appropriate endpoints.
 @param issuerUrl Authorization server's Issuer Url
 @param logoutUrl Url to logout the user session at the authorization server
 */
- (nullable instancetype) initWithIssuerUrl:(nonnull NSURL *) issuerUrl
                                  logoutUrl:(nullable NSURL *) logoutUrl;

/*! @brief Is already authorized
 @return YES if already authorized.
 */
- (BOOL) isAuthorized;

/*! @brief Set auth state with tokens.
 */
- (void)setAuthState:(nullable OIDAuthState *)authState;

/*! @brief load state with tokens from device keychain
 */
- (void) loadState;

/*! @brief save state with tokens to device keychain
 */
- (void) saveState;

/*! @brief SignIn user to authorization server and obtain the tokens.
 This will perform OAuth authorization code flow utilizing PKCE the Safari ViewController based on security best practices.
 @param clientId The oauth clientId.
 @param clientSecret oauth client secret.
 @param scopes oauth / openid connect scopes.
 @param redirectURL redirect url as configured in oauth client application.
 @param presentingVC presenting view controller of your app.
 @param callbackBlock block called after sigin. This will have success flag or error.
 @return A @c OIDAuthorizationFlowSession instance which will terminate when it
 receives a @c OIDAuthorizationFlowSession.cancel message, or after processing a
 @c OIDAuthorizationFlowSession.resumeAuthorizationFlowWithURL: message.
 Part of the oauth authorization flow will be done after app reentry after user authentication using @c OIDAuthorizationFlowSession.
 */
- (nullable id<OIDAuthorizationFlowSession>) signInWithClientId:(nonnull NSString *) clientId
                                               clientSecret:(nullable NSString *) clientSecret
                                                     scopes:(nullable NSArray<NSString *> *) scopes
                                                redirectURL:(nullable NSURL *) redirectURL
                                   presentingViewController:(nonnull UIViewController *) presentingVC
                                                  withBlock:(nonnull MFSignInCompletionCallback) callbackBlock;

/*! @brief SignIn user to authorization server and obtain the tokens.
 Perform an authorization flow using a generic flow shim.
 This will perform OAuth authorization code flow utilizing PKCE the Safari ViewController based on security best practices.
 @param clientId The oauth clientId.
 @param clientSecret Oauth client secret.
 @param scopes Oauth / Openid connect scopes.
 @param redirectURL Redirect url as configured in oauth client application.
 @param responseType This identifies the oauth grant types
 @param additionalParameters Additional parameters that can be put on authorization request.
 @param presentingVC Presenting view controller of your app.
 @param callbackBlock Block called after sigin. This will have success flag or error.
 @return A @c OIDAuthorizationFlowSession instance which will terminate when it
 receives a @c OIDAuthorizationFlowSession.cancel message, or after processing a
 @c OIDAuthorizationFlowSession.resumeAuthorizationFlowWithURL: message.
 */
- (nullable id<OIDAuthorizationFlowSession>) signInWithClientId:(nonnull NSString *) clientId
                                           clientSecret:(nullable NSString *) clientSecret
                                                 scopes:(nullable NSArray<NSString *> *) scopes
                                            redirectURL:(nullable NSURL *) redirectURL
                                           responseType:(nullable NSString *) responseType
                                   additionalParameters:(nullable NSMutableDictionary<NSString *, NSString *> *) additionalParameters
                               presentingViewController:(nonnull UIViewController *) presentingVC
                                              withBlock:(nonnull MFSignInCompletionCallback) callbackBlock;

/*! @brief Get user info response data from UserInfo endpoint.
 This uses access token as bearer on the call and will automatically refresh the token, if needed.
 @param completionHandler block to be executed after the completion of the operation.
 It will have the response data from the request or error.
 */
- (void) getUserInfoResponse:(nonnull MFResponseCallback) completionHandler;

/*! @brief Get user info from UserInfo endpoint.
 This uses access token as bearer on the call and will automatically refresh the token, if needed.
 @param completionHandler block to be executed after the completion of the operation.
 It will have the user data in a dictionary from the request or error.
 */
- (void) getUserInfo:(nonnull MFUserInfoCallback) completionHandler;

/*! @brief Performs the action specified in the callback with the access or id token.
 It will refresh the token, if needed.
 @param action Callback block which is executed after the operation with a response or error.
 */
- (void) performActionWithFreshToken:(nonnull MFAuthStateAction) action;

/*! @brief A convenience method that performs a basic http request and calls back with the response.
 It will refresh the token, if needed.
 @param requestUrl Url to perform the http Get request.
 @param handler Callback block which is executed with the fresh (if expired) tokens.
 */
- (void) performHTTPRequestWithFreshToken:(nonnull NSURL *) requestUrl
                        withCompletionHandler:(nonnull MFResponseCallback) handler;

/*! @brief Get the access token.
 */
- (nullable NSString *) getAccessToken;

/*! @brief Get the access token expiration time string in local time.
 */
- (nullable NSString *) getAccessTokenExpiryTimeString;

/*! @brief Logout user session at the authorization server.
 This deletes all the tokens from the device,
 Revokes Refresh token on authorization server, and
 Logs out the user session at the authorization server.
 @discussion Note, This will invalidate the session in system browser and/or @c SafariViewController
 thus, next time during signIn, the user will be prompted for credentials to authenticate.
 This will not, however, affect the already authorized apps.
 */
- (void) logout:(nonnull UIViewController *) presentingVC;

/*! @brief Revoke the Refresh Token.
 */
- (void) revokeToken:(nonnull NSURL *) revocationEndpoint;


@end
